import { isFunction } from 'underscore';

module.exports = require('./AssetView').extend({
  events: {
    'click [data-toggle=asset-remove]': 'onRemove',
    click: 'onClick',
    dblclick: 'onDblClick'
  },
  getPreview() {
    const pfx = this.pfx;
    const src = this.model.get('src');
    return `
      <div class="${pfx}preview" style="background-image: url('${src}');"></div>
      <div class="${pfx}preview-bg ${this.ppfx}checker-bg"></div>
    `;
  },
  getInfo() {
    const pfx = this.pfx;
    const model = this.model;
    let name = model.get('name');
    const width = model.get('width');
    const height = model.get('height');
    const unit = model.get('unitDim');
    const dim = width && height ? `${width}x${height}${unit}` : '';
    name = name || model.getFilename();
    return `
      <div class="${pfx}name">${name}</div>
      <div class="${pfx}dimensions">${dim}</div>
    `;
  },
  init(o) {
    const pfx = this.pfx;
    this.className += ` ${pfx}asset-image`;
  },
  onClick() {
    const onClick = this.config.onClick;
    const model = this.model;
    this.collection.trigger('deselectAll');
    this.$el.addClass(`${this.pfx}highlight`);

    if (isFunction(onClick)) {
      onClick(model);
    } else {
      this.updateTarget(this.collection.target);
    }
  },
  onDblClick() {
    const { em, model } = this;
    const onDblClick = this.config.onDblClick;

    if (isFunction(onDblClick)) {
      onDblClick(model);
    } else {
      this.updateTarget(this.collection.target);
      em && em.get('Modal').close();
    }
    const onSelect = this.collection.onSelect;
    isFunction(onSelect) && onSelect(model);
  },
  onRemove(e) {
    e.stopImmediatePropagation();
    this.model.collection.remove(this.model);
  }
});
