import { isElement } from 'underscore';

module.exports = () => {
  let c = {};
  const defaults = require('./config/config');
  const Blocks = require('./model/Blocks');
  const BlockCategories = require('./model/Categories');
  const BlocksView = require('./view/BlocksView');
  let blocks;
  let blocksVisible;
  let blocksView;
  let categories = [];

  return {
    name: 'BlockManager',
    init(config) {
      c = config || {};
      const em = c.em;

      for (const name in defaults) {
        if (!(name in c)) {
          c[name] = defaults[name];
        }
      }
      blocks = new Blocks([]);
      blocksVisible = new Blocks([]);
      (categories = new BlockCategories()),
        (blocksView = new BlocksView(
          {
            collection: blocksVisible,
            categories
          },
          c
        ));
      blocks.listenTo(blocks, 'add', model => {
        blocksVisible.add(model);
        if (em && em.trigger) {
          em.trigger('block:add', model);
        }
      });
      blocks.listenTo(blocks, 'remove', model => {
        blocksVisible.remove(model);
        if (em && em.trigger) {
          em.trigger('block:remove', model);
        }
      });
      blocks.listenTo(blocks, 'reset', coll => {
        blocksVisible.reset(coll.models);
      });
      return this;
    },
    getConfig() {
      return c;
    },
    onLoad() {
      const blocks = this.getAll();
      if (blocks && !blocks.length) {
        blocks.reset(c.blocks);
      }
    },
    postRender() {
      const elTo = this.getConfig().appendTo;
      if (elTo) {
        const el = isElement(elTo) ? elTo : document.querySelector(elTo);
        el.appendChild(this.render());
      }
    },
    add(id, opts) {
      const obj = opts || {};
      obj.id = id;
      return blocks.add(obj);
    },
    get(id) {
      return blocks.get(id);
    },
    getAll() {
      return blocks;
    },
    getAllVisible() {
      return blocksVisible;
    },
    remove(id) {
      return blocks.remove(id);
    },
    getCategories() {
      return categories;
    },
    getContainer() {
      return blocksView.el;
    },
    render(blocks = false) {
      const blocksVar = blocks;
      const toRender = blocksVar || this.getAll().models;
      if (!blocksView.rendered) {
        blocksView.render();
        blocksView.rendered = 1;
      }
      blocksView.collection.reset(toRender);
      return this.getContainer();
    }
  };
};
