import { on, off, hasDnd, getElement } from 'utils/mixins';
import Droppable from 'utils/Droppable';

module.exports = () => {
  let c = {};
  const defaults = require('./config/config');
  const Canvas = require('./model/Canvas');
  let CanvasView = require('./view/CanvasView');
  let canvas;
  let frameRect;
  return {
    getCanvasView() {
      return CanvasView;
    },
    name: 'Canvas',
    init(config) {
      c = config || {};
      for (const name in defaults) {
        if (!(name in c)) c[name] = defaults[name];
      }
      const ppfx = c.pStylePrefix;
      if (ppfx) {
        c.stylePrefix = ppfx + c.stylePrefix;
      }
      canvas = new Canvas(config);
      CanvasView = new CanvasView({
        model: canvas,
        config: c
      });
      const cm = c.em.get('DomComponents');
      if (cm) this.setWrapper(cm);
      this.startAutoscroll = this.startAutoscroll.bind(this);
      this.stopAutoscroll = this.stopAutoscroll.bind(this);
      this.autoscroll = this.autoscroll.bind(this);
      return this;
    },
    getConfig() {
      return c;
    },
    setWrapper(wrp) {
      canvas.set('wrapper', wrp);
    },
    getElement() {
      return CanvasView.el;
    },
    getFrameEl() {
      return CanvasView.frame.el;
    },
    getWindow() {
      return this.getFrameEl().contentWindow;
    },
    getDocument() {
      return this.getFrameEl().contentDocument;
    },
    getBody() {
      const doc = this.getDocument();
      return doc && doc.body;
    },
    getWrapperEl() {
      const body = this.getBody();
      return body && body.querySelector('#wrapper');
    },
    getToolsEl() {
      return CanvasView.toolsEl;
    },
    getHighlighter() {
      return CanvasView.hlEl;
    },
    getBadgeEl() {
      return CanvasView.badgeEl;
    },
    getPlacerEl() {
      return CanvasView.placerEl;
    },
    getGhostEl() {
      return CanvasView.ghostEl;
    },
    getToolbarEl() {
      return CanvasView.toolbarEl;
    },
    getResizerEl() {
      return CanvasView.resizerEl;
    },
    getOffsetViewerEl() {
      return CanvasView.offsetEl;
    },
    getFixedOffsetViewerEl() {
      return CanvasView.fixedOffsetEl;
    },
    render() {
      return CanvasView.render().el;
    },
    getOffset() {
      const frameOff = this.offset(this.getFrameEl());
      const canvasOff = this.offset(this.getElement());
      return {
        top: frameOff.top - canvasOff.top,
        left: frameOff.left - canvasOff.left
      };
    },
    offset(el) {
      return CanvasView.offset(el);
    },
    setCustomBadgeLabel(f) {
      c.customBadgeLabel = f;
    },
    getElementPos(el, opts) {
      return CanvasView.getElementPos(el, opts);
    },
    getTargetToElementDim(target, element, options) {
      const opts = options || {};
      const canvasPos = CanvasView.getPosition();
      if (!canvasPos) return;
      const pos = opts.elPos || CanvasView.getElementPos(element);
      const toRight = options.toRight || 0;
      const targetHeight = opts.targetHeight || target.offsetHeight;
      const targetWidth = opts.targetWidth || target.offsetWidth;
      const eventToTrigger = opts.event || null;
      const elTop = pos.top - targetHeight;
      let elLeft = pos.left;
      elLeft += toRight ? pos.width : 0;
      elLeft = toRight ? elLeft - targetWidth : elLeft;
      const leftPos = elLeft < canvasPos.left ? canvasPos.left : elLeft;
      let topPos = elTop < canvasPos.top ? canvasPos.top : elTop;
      topPos = topPos > pos.top + pos.height ? pos.top + pos.height : topPos;
      const result = {
        top: topPos,
        left: leftPos,
        elementTop: pos.top,
        elementLeft: pos.left,
        elementWidth: pos.width,
        elementHeight: pos.height,
        targetWidth: target.offsetWidth,
        targetHeight: target.offsetHeight,
        canvasTop: canvasPos.top,
        canvasLeft: canvasPos.left
      };
      if (eventToTrigger && c.em) {
        c.em.trigger(eventToTrigger, result);
      }
      return result;
    },
    getMouseRelativePos(e, options) {
      const opts = options || {};
      let addTop = 0;
      let addLeft = 0;
      const subWinOffset = opts.subWinOffset;
      const doc = e.target.ownerDocument;
      const win = doc.defaultView || doc.parentWindow;
      const frame = win.frameElement;
      const yOffset = subWinOffset ? win.pageYOffset : 0;
      const xOffset = subWinOffset ? win.pageXOffset : 0;
      if (frame) {
        const frameRect = frame.getBoundingClientRect();
        addTop = frameRect.top || 0;
        addLeft = frameRect.left || 0;
      }
      return {
        y: e.clientY + addTop - yOffset,
        x: e.clientX + addLeft - xOffset
      };
    },
    getMouseRelativeCanvas(e, options) {
      const opts = options || {};
      const frame = this.getFrameEl();
      const body = this.getBody();
      const addTop = frame.offsetTop || 0;
      const addLeft = frame.offsetLeft || 0;
      const yOffset = body.scrollTop || 0;
      const xOffset = body.scrollLeft || 0;
      return {
        y: e.clientY + addTop + yOffset,
        x: e.clientX + addLeft + xOffset
      };
    },
    hasFocus() {
      return this.getDocument().hasFocus();
    },
    isInputFocused() {
      const doc = this.getDocument();
      const toIgnore = ['body', ...this.getConfig().notTextable];
      const focused = doc && doc.activeElement;
      return focused && !toIgnore.some(item => focused.matches(item));
    },
    scrollTo(el, opts = {}) {
      const elem = getElement(el);
      const cv = this.getCanvasView();

      if (!cv.isElInViewport(elem) || opts.force) {
        elem.scrollIntoView(opts);
      }
    },
    startAutoscroll() {
      this.dragging = 1;
      const toListen = this.getScrollListeners();
      frameRect = CanvasView.getFrameOffset(1);
      setTimeout(() => {
        on(toListen, 'mousemove', this.autoscroll);
        on(toListen, 'mouseup', this.stopAutoscroll);
      }, 0);
    },
    autoscroll(e) {
      e.preventDefault();
      if (this.dragging) {
        const frameWindow = this.getFrameEl().contentWindow;
        const actualTop = frameWindow.document.body.scrollTop;
        let nextTop = actualTop;
        const clientY = e.clientY;
        const limitTop = 50;
        const limitBottom = frameRect.height - limitTop;
        if (clientY < limitTop) {
          nextTop -= limitTop - clientY;
        }
        if (clientY > limitBottom) {
          nextTop += clientY - limitBottom;
        }
        frameWindow.scrollTo(0, nextTop);
      }
    },
    stopAutoscroll() {
      this.dragging = 0;
      const toListen = this.getScrollListeners();
      off(toListen, 'mousemove', this.autoscroll);
      off(toListen, 'mouseup', this.stopAutoscroll);
    },
    getScrollListeners() {
      return [this.getFrameEl().contentWindow, this.getElement()];
    },
    postRender() {
      if (hasDnd(c.em)) this.droppable = new Droppable(c.em);
    },
    getFrameWrapperEl() {
      return CanvasView.frame.getWrapper();
    }
  };
};
