import { isUndefined } from 'underscore';

module.exports = () => {
  let c = {};
  const defaults = require('./config/config');
  const gHtml = require('./model/HtmlGenerator');
  const gCss = require('./model/CssGenerator');
  const gJson = require('./model/JsonGenerator');
  const gJs = require('./model/JsGenerator');
  const eCM = require('./model/CodeMirrorEditor');
  const editorView = require('./view/EditorView');
  const generators = {};
  const defGenerators = {};
  const viewers = {};
  const defViewers = {};
  const defaultViewer = 'CodeMirror';
  return {
    getConfig() {
      return c;
    },
    config: c,
    EditorView: editorView,
    name: 'CodeManager',
    init(config) {
      c = config || {};
      for (const name in defaults) {
        if (!(name in c)) c[name] = defaults[name];
      }
      const ppfx = c.pStylePrefix;
      if (ppfx) c.stylePrefix = ppfx + c.stylePrefix;
      defGenerators.html = new gHtml();
      defGenerators.css = new gCss();
      defGenerators.json = new gJson();
      defGenerators.js = new gJs();
      defViewers.CodeMirror = new eCM();
      this.loadDefaultGenerators().loadDefaultViewers();

      return this;
    },
    addGenerator(id, generator) {
      generators[id] = generator;
      return this;
    },
    getGenerator(id) {
      return generators[id] || null;
    },
    getGenerators() {
      return generators;
    },
    addViewer(id, viewer) {
      viewers[id] = viewer;
      return this;
    },
    getViewer(id) {
      return viewers[id] || null;
    },
    getViewers() {
      return viewers;
    },
    createViewer(opts = {}) {
      const type = !isUndefined(opts.type) ? opts.type : defaultViewer;
      const viewer = this.getViewer(type) && this.getViewer(type).clone();
      const cont = document.createElement('div');
      const txtarea = document.createElement('textarea');
      cont.appendChild(txtarea);
      viewer.set(opts);
      viewer.init(txtarea);
      viewer.setElement(cont);
      return viewer;
    },
    updateViewer(viewer, code) {
      viewer.setContent(code);
    },
    getCode(model, genId, opt = {}) {
      opt.em = c.em;
      const generator = this.getGenerator(genId);
      return generator ? generator.build(model, opt) : '';
    },
    loadDefaultGenerators() {
      for (const id in defGenerators) {
        this.addGenerator(id, defGenerators[id]);
      }
      return this;
    },
    loadDefaultViewers() {
      for (const id in defViewers) {
        this.addViewer(id, defViewers[id]);
      }
      return this;
    }
  };
};
