import Backbone from 'backbone';

const CodeMirror = require('codemirror/lib/codemirror');
const htmlMode = require('codemirror/mode/htmlmixed/htmlmixed');
const cssMode = require('codemirror/mode/css/css');
const formatting = require('codemirror-formatting');

module.exports = Backbone.Model.extend({
  defaults: {
    input: '',
    label: '',
    codeName: '',
    theme: '',
    readOnly: true,
    lineNumbers: true
  },
  init(el) {
    this.editor = CodeMirror.fromTextArea(el, {
      dragDrop: false,
      lineWrapping: true,
      mode: this.get('codeName'),
      ...this.attributes
    });
    this.element = el;
    return this;
  },
  getEditor() {
    return this.editor;
  },
  getElement() {
    return this.element;
  },
  setElement(el) {
    this.element = el;
    return this;
  },
  refresh() {
    this.getEditor().refresh();
    return this;
  },
  focus() {
    this.getEditor().focus();
    return this;
  },
  getContent() {
    const ed = this.getEditor();
    return ed && ed.getValue();
  },
  setContent(v) {
    if (!this.editor) return;
    this.editor.setValue(v);
    if (this.editor.autoFormatRange) {
      CodeMirror.commands.selectAll(this.editor);
      this.editor.autoFormatRange(
        this.editor.getCursor(true),
        this.editor.getCursor(false)
      );
      CodeMirror.commands.goDocStart(this.editor);
    }
  }
});
