import { extend } from 'underscore';
import Backbone from 'backbone';

module.exports = Backbone.Model.extend({
  mapModel(model) {
    let code = '';
    const script = model.get('script');
    const type = model.get('type');
    const comps = model.get('components');
    const id = model.getId();
    if (script) {
      let attr = model.get('attributes');
      attr = extend({}, attr, { id });
      model.set('attributes', attr);
      const scrStr = model.getScriptString();
      if (model.get('scriptUpdated')) {
        this.mapJs[`${type}-${id}`] = { ids: [id], code: scrStr };
      } else {
        const mapType = this.mapJs[type];
        if (mapType) {
          mapType.ids.push(id);
        } else {
          this.mapJs[type] = { ids: [id], code: scrStr };
        }
      }
    }
    comps.each(function(model) {
      code += this.mapModel(model);
    }, this);
    return code;
  },
  build(model) {
    this.mapJs = {};
    this.mapModel(model);
    let code = '';
    for (const type in this.mapJs) {
      const mapType = this.mapJs[type];
      const ids = `#${mapType.ids.join(', #')}`;
      code += `
        var items = document.querySelectorAll('${ids}');
        for (var i = 0, len = items.length; i < len; i++) {
          (function(){${mapType.code}}.bind(items[i]))();
        }`;
    }
    return code;
  }
});
