import _ from 'underscore';
import Backbone from 'backbone';

const SelectPosition = require('./SelectPosition');

const $ = Backbone.$;

module.exports = _.extend({}, SelectPosition, {
  init(opt) {
    _.bindAll(this, 'startDraw', 'draw', 'endDraw', 'rollback');
    this.config = opt || {};
    this.hType = this.config.newFixedH ? 'height' : 'min-height';
    this.allowDraw = 1;
  },
  enable(...args) {
    SelectPosition.enable.apply(this, args);
    this.$wr.css('cursor', 'crosshair');
    if (this.allowDraw) this.$wr.on('mousedown', this.startDraw);
    this.ghost = this.canvas.getGhostEl();
  },
  startDraw(e) {
    e.preventDefault();
    this.stopSelectPosition();
    this.ghost.style.display = 'block';
    this.frameOff = this.getOffsetDim();
    this.startPos = {
      top: e.pageY + this.frameOff.top,
      left: e.pageX + this.frameOff.left
    };
    this.isDragged = false;
    this.tempComponent = { style: {} };
    this.beforeDraw(this.tempComponent);
    this.updateSize(this.startPos.top, this.startPos.left, 0, 0);
    this.toggleEvents(1);
  },
  toggleEvents(enable) {
    const method = enable ? 'on' : 'off';
    this.$wr[method]('mousemove', this.draw);
    this.$wr[method]('mouseup', this.endDraw);
    this.$canvas[method]('mousemove', this.draw);
    $(document)[method]('mouseup', this.endDraw);
    $(document)[method]('keypress', this.rollback);
  },
  draw(e) {
    this.isDragged = true;
    this.updateComponentSize(e);
  },
  endDraw(e) {
    this.toggleEvents();
    let model = {};
    if (this.isDragged) {
      this.updateComponentSize(e);
      this.setRequirements(this.tempComponent);
      const lp = this.sorter.lastPos;
      model = this.create(
        this.sorter.target,
        this.tempComponent,
        lp.index,
        lp.method
      );
      this.sorter.prevTarget = null;
    }
    this.ghost.style.display = 'none';
    this.startSelectPosition();
    this.afterDraw(model);
  },
  create(target, component, index, method, opts) {
    index = method === 'after' ? index + 1 : index;
    const opt = opts || {};
    const $trg = $(target);
    const trgModel = $trg.data('model');
    const trgCollection = $trg.data('collection');
    const droppable = trgModel ? trgModel.get('droppable') : 1;
    opt.at = index;
    if (trgCollection && droppable) return trgCollection.add(component, opt);
    console.warn('Invalid target position');
  },
  setRequirements(component) {
    const c = this.config;
    const compStl = component.style;
    if (compStl.width.replace(/\D/g, '') < c.minComponentW)
      compStl.width = `${c.minComponentW}px`;
    if (compStl[this.hType].replace(/\D/g, '') < c.minComponentH)
      compStl[this.hType] = `${c.minComponentH}px`;
    if (c.newFixedH) compStl.overflow = 'auto';
    if (!this.absoluteMode) {
      delete compStl.left;
      delete compStl.top;
    } else compStl.position = 'absolute';
    const lp = this.sorter.lastPos;
    if (this.nearFloat(lp.index, lp.method, this.sorter.lastDims))
      compStl.float = 'left';
    if (
      this.config.firstCentered &&
      this.getCanvasWrapper() == this.sorter.target
    ) {
      compStl.margin = '0 auto';
    }
    return component;
  },
  updateComponentSize(e) {
    const y = e.pageY + this.frameOff.top;
    const x = e.pageX + this.frameOff.left;
    const start = this.startPos;
    let top = start.top;
    let left = start.left;
    let height = y - top;
    let width = x - left;
    if (x < left) {
      left = x;
      width = start.left - x;
    }
    if (y < top) {
      top = y;
      height = start.top - y;
    }
    this.updateSize(top, left, width, height);
  },
  updateSize(top, left, width, height) {
    const u = 'px';
    const ghStl = this.ghost.style;
    const compStl = this.tempComponent.style;
    ghStl.top = compStl.top = top + u;
    ghStl.left = compStl.left = left + u;
    ghStl.width = compStl.width = width + u;
    ghStl[this.hType] = compStl[this.hType] = height + u;
  },
  rollback(e, force) {
    const key = e.which || e.keyCode;
    if (key == this.config.ESCAPE_KEY || force) {
      this.isDragged = false;
      this.endDraw();
    }
  },
  beforeDraw(component) {
    component.editable = false; // set this component editable
  },
  afterDraw(model) {},
  run(editor, sender, opts) {
    this.editor = editor;
    this.sender = sender;
    this.$wr = this.$wrapper;
    this.enable();
  },
  stop() {
    this.stopSelectPosition();
    this.$wrapper.css('cursor', '');
    this.$wrapper.unbind();
  }
});
