import _ from 'underscore';
import Backbone from 'backbone';

const SelectComponent = require('./SelectComponent');

const $ = Backbone.$;

module.exports = _.extend({}, SelectComponent, {
  init(o) {
    _.bindAll(this, 'startDelete', 'stopDelete', 'onDelete');
    this.hoverClass = `${this.pfx}hover-delete`;
    this.badgeClass = `${this.pfx}badge-red`;
  },

  enable() {
    this.$el
      .find('*')
      .mouseover(this.startDelete)
      .mouseout(this.stopDelete)
      .click(this.onDelete);
  },
  startDelete(e) {
    e.stopPropagation();
    const $this = $(e.target);
    if ($this.data('model').get('removable')) {
      $this.addClass(this.hoverClass);
      this.attachBadge($this.get(0));
    }
  },
  stopDelete(e) {
    e.stopPropagation();
    const $this = $(e.target);
    $this.removeClass(this.hoverClass);
    if (this.badge) this.badge.css({ left: -1000, top: -1000 });
  },
  onDelete(e) {
    e.stopPropagation();
    const $this = $(e.target);
    if (!$this.data('model').get('removable')) return;
    $this.data('model').destroy();
    this.removeBadge();
    this.clean();
  },
  updateBadgeLabel(model) {
    this.badge.html(`Remove ${model.getName()}`);
  }
});
