import _ from 'underscore';
import Backbone from 'backbone';
import { on, off } from 'utils/mixins';

const SelectComponent = require('./SelectComponent');
const SelectPosition = require('./SelectPosition');

const $ = Backbone.$;
module.exports = _.extend({}, SelectPosition, SelectComponent, {
  init(o) {
    SelectComponent.init.apply(this, arguments);
    _.bindAll(this, 'initSorter', 'rollback', 'onEndMove');
    this.opt = o;
    this.hoverClass = `${this.ppfx}highlighter-warning`;
    this.badgeClass = `${this.ppfx}badge-warning`;
    this.noSelClass = `${this.ppfx}no-select`;
  },
  enable(...args) {
    SelectComponent.enable.apply(this, args);
    this.getBadgeEl().addClass(this.badgeClass);
    this.getHighlighterEl().addClass(this.hoverClass);
    const wp = this.$wrapper;
    wp.css('cursor', 'move');
    wp.on('mousedown', this.initSorter);
    wp.addClass(this.noSelClass);
  },
  toggleClipboard() {},
  initSorter(e) {
    const el = $(e.target).data('model');
    const drag = el.get('draggable');
    if (!drag) {
      return;
    }
    this.cacheEl = null;
    this.startSelectPosition(e.target, this.frameEl.contentDocument);
    this.sorter.draggable = drag;
    this.sorter.onEndMove = this.onEndMove.bind(this);
    this.stopSelectComponent();
    this.$wrapper.off('mousedown', this.initSorter);
    on(this.getContentWindow(), 'keydown', this.rollback);
  },
  initSorterFromModel(model) {
    const drag = model.get('draggable');
    if (!drag) return;
    this.cacheEl = null;
    const el = model.view.el;
    this.startSelectPosition(el, this.frameEl.contentDocument);
    this.sorter.draggable = drag;
    this.sorter.onEndMove = this.onEndMoveFromModel.bind(this);
    this.stopSelectComponent();
    on(this.getContentWindow(), 'keydown', this.rollback);
  },
  initSorterFromModels(models) {
    this.cacheEl = null;
    const lastModel = models[models.length - 1];
    const doc = this.frameEl.contentDocument;
    this.startSelectPosition(lastModel.view.el, doc);
    this.sorter.draggable = lastModel.get('draggable');
    this.sorter.toMove = models;
    this.sorter.onEndMove = this.onEndMoveFromModel.bind(this);
    this.stopSelectComponent();
    on(this.getContentWindow(), 'keydown', this.rollback);
  },
  onEndMoveFromModel() {
    off(this.getContentWindow(), 'keydown', this.rollback);
  },
  onEndMove() {
    this.enable();
    off(this.getContentWindow(), 'keydown', this.rollback);
  },
  onSelect() {},
  rollback(e, force) {
    const key = e.which || e.keyCode;
    if (key == this.opt.ESCAPE_KEY || force) {
      this.sorter.moved = false;
      this.sorter.endMove();
    }
  },
  getBadgeEl() {
    if (!this.$badge) this.$badge = $(this.getBadge());
    return this.$badge;
  },
  getHighlighterEl() {
    if (!this.$hl) this.$hl = $(this.canvas.getHighlighter());
    return this.$hl;
  },
  stop(...args) {
    SelectComponent.stop.apply(this, args);
    this.getBadgeEl().removeClass(this.badgeClass);
    this.getHighlighterEl().removeClass(this.hoverClass);
    const wp = this.$wrapper;
    wp.css('cursor', '')
      .unbind()
      .removeClass(this.noSelClass);
  }
});
