import Backbone from 'backbone';

const $ = Backbone.$;
module.exports = {
  getOffsetMethod(state) {
    const method = state || '';
    return `get${method}OffsetViewerEl`;
  },
  run(editor, sender, opts) {
    const opt = opts || {};
    const state = opt.state || '';
    const config = editor.getConfig();
    if (
      !config.showOffsets ||
      (!config.showOffsetsSelected && state == 'Fixed')
    ) {
      return;
    }
    const canvas = editor.Canvas;
    const el = opt.el || '';
    const pos = opt.elPos || canvas.getElementPos(el);
    const style = window.getComputedStyle(el);
    const ppfx = this.ppfx;
    const stateVar = `${state}State`;
    const method = this.getOffsetMethod(state);
    const offsetViewer = canvas[method]();
    offsetViewer.style.display = 'block';
    let marginT = this[`marginT${state}`];
    let marginB = this[`marginB${state}`];
    let marginL = this[`marginL${state}`];
    let marginR = this[`marginR${state}`];
    let padT = this[`padT${state}`];
    let padB = this[`padB${state}`];
    let padL = this[`padL${state}`];
    let padR = this[`padR${state}`];
    if (!this[stateVar]) {
      const stateLow = state.toLowerCase();
      const marginName = `${stateLow}margin-v`;
      const paddingName = `${stateLow}padding-v`;
      const marginV = $(`<div class="${ppfx}marginName">`).get(0);
      const paddingV = $(`<div class="${ppfx}paddingName">`).get(0);
      const marginEls = `${ppfx + marginName}-el`;
      const paddingEls = `${ppfx + paddingName}-el`;
      const fullMargName = `${marginEls} ${ppfx + marginName}`;
      const fullPadName = `${paddingEls} ${ppfx + paddingName}`;
      marginT = $(`<div class="${fullMargName}-top"></div>`).get(0);
      marginB = $(`<div class="${fullMargName}-bottom"></div>`).get(0);
      marginL = $(`<div class="${fullMargName}-left"></div>`).get(0);
      marginR = $(`<div class="${fullMargName}-right"></div>`).get(0);
      padT = $(`<div class="${fullPadName}-top"></div>`).get(0);
      padB = $(`<div class="${fullPadName}-bottom"></div>`).get(0);
      padL = $(`<div class="${fullPadName}-left"></div>`).get(0);
      padR = $(`<div class="${fullPadName}-right"></div>`).get(0);
      this[`marginT${state}`] = marginT;
      this[`marginB${state}`] = marginB;
      this[`marginL${state}`] = marginL;
      this[`marginR${state}`] = marginR;
      this[`padT${state}`] = padT;
      this[`padB${state}`] = padB;
      this[`padL${state}`] = padL;
      this[`padR${state}`] = padR;
      marginV.appendChild(marginT);
      marginV.appendChild(marginB);
      marginV.appendChild(marginL);
      marginV.appendChild(marginR);
      paddingV.appendChild(padT);
      paddingV.appendChild(padB);
      paddingV.appendChild(padL);
      paddingV.appendChild(padR);
      offsetViewer.appendChild(marginV);
      offsetViewer.appendChild(paddingV);
      this[stateVar] = '1';
    }
    const unit = 'px';
    const marginLeftSt = style.marginLeft.replace(unit, '');
    const marginTopSt = parseInt(style.marginTop.replace(unit, ''), 10);
    const marginBottomSt = parseInt(style.marginBottom.replace(unit, ''), 10);
    const mtStyle = marginT.style;
    const mbStyle = marginB.style;
    const mlStyle = marginL.style;
    const mrStyle = marginR.style;
    const ptStyle = padT.style;
    const pbStyle = padB.style;
    const plStyle = padL.style;
    const prStyle = padR.style;
    const posLeft = parseInt(pos.left, 10);
    mtStyle.height = style.marginTop;
    mtStyle.width = style.width;
    mtStyle.top = pos.top - style.marginTop.replace(unit, '') + unit;
    mtStyle.left = posLeft + unit;
    mbStyle.height = style.marginBottom;
    mbStyle.width = style.width;
    mbStyle.top = pos.top + pos.height + unit;
    mbStyle.left = posLeft + unit;
    const marginSideH = pos.height + marginTopSt + marginBottomSt + unit;
    const marginSideT = pos.top - marginTopSt + unit;
    mlStyle.height = marginSideH;
    mlStyle.width = style.marginLeft;
    mlStyle.top = marginSideT;
    mlStyle.left = posLeft - marginLeftSt + unit;
    mrStyle.height = marginSideH;
    mrStyle.width = style.marginRight;
    mrStyle.top = marginSideT;
    mrStyle.left = posLeft + pos.width + unit;
    const padTop = parseInt(style.paddingTop.replace(unit, ''), 10);
    ptStyle.height = style.paddingTop;
    ptStyle.width = style.width;
    ptStyle.top = pos.top + unit;
    ptStyle.left = posLeft + unit;
    const padBot = parseInt(style.paddingBottom.replace(unit, ''), 10);
    pbStyle.height = style.paddingBottom;
    pbStyle.width = style.width;
    pbStyle.top = pos.top + pos.height - padBot + unit;
    pbStyle.left = posLeft + unit;
    const padSideH = pos.height - padBot - padTop + unit;
    const padSideT = pos.top + padTop + unit;
    plStyle.height = padSideH;
    plStyle.width = style.paddingLeft;
    plStyle.top = padSideT;
    plStyle.left = pos.left + unit;
    const padRight = parseInt(style.paddingRight.replace(unit, ''), 10);
    prStyle.height = padSideH;
    prStyle.width = style.paddingRight;
    prStyle.top = padSideT;
    prStyle.left = pos.left + pos.width - padRight + unit;
  },
  stop(editor, sender, opts) {
    const opt = opts || {};
    const state = opt.state || '';
    const method = this.getOffsetMethod(state);
    const canvas = editor.Canvas;
    const offsetViewer = canvas[method]();
    offsetViewer.style.display = 'none';
  }
};
