import _ from 'underscore';
import Backbone from 'backbone';

module.exports = Backbone.View.extend({
  template: _.template(`
    <div class="<%= ppfx %>device-label"><%= deviceLabel %></div>
    <div class="<%= ppfx %>field <%= ppfx %>select">
      <span id="<%= ppfx %>input-holder">
        <select class="<%= ppfx %>devices"></select>
      </span>
      <div class="<%= ppfx %>sel-arrow">
        <div class="<%= ppfx %>d-s-arrow"></div>
      </div>
    </div>
    <button style="display:none" class="<%= ppfx %>add-trasp">+</button>`),

  events: {
    change: 'updateDevice'
  },
  initialize(o) {
    this.config = o.config || {};
    this.em = this.config.em;
    this.ppfx = this.config.pStylePrefix || '';
    this.events[`click .${this.ppfx}add-trasp`] = this.startAdd;
    this.listenTo(this.em, 'change:device', this.updateSelect);
    this.delegateEvents();
  },
  startAdd() {},
  updateDevice() {
    const em = this.em;
    if (em) {
      const devEl = this.devicesEl;
      const val = devEl ? devEl.val() : '';
      em.set('device', val);
    }
  },
  updateSelect() {
    const em = this.em;
    const devEl = this.devicesEl;
    if (em && em.getDeviceModel && devEl) {
      const device = em.getDeviceModel();
      const name = device ? device.get('name') : '';
      devEl.val(name);
    }
  },
  getOptions() {
    let result = '';
    this.collection.each(device => {
      const name = device.get('name');
      result += `<option value="${name}">${name}</option>`;
    });
    return result;
  },
  render() {
    const pfx = this.ppfx;
    this.$el.html(
      this.template({
        ppfx: pfx,
        deviceLabel: this.config.deviceLabel
      })
    );
    this.devicesEl = this.$el.find(`.${pfx}devices`);
    this.devicesEl.append(this.getOptions());
    this.el.className = `${pfx}devices-c`;
    return this;
  }
});
