const Component = require('./Component');

module.exports = Component.extend(
  {
    defaults: {
      ...Component.prototype.defaults,
      type: 'image',
      tagName: 'img',
      src: '',
      void: 1,
      droppable: 0,
      editable: 1,
      highlightable: 0,
      resizable: 1,
      traits: ['alt'],
      file: ''
    },
    initialize(o, opt) {
      Component.prototype.initialize.apply(this, arguments);
      const attr = this.get('attributes');
      if (attr.src) {
        this.set('src', attr.src);
      }
    },

    initToolbar(...args) {
      Component.prototype.initToolbar.apply(this, args);
      const em = this.em;
      if (em) {
        const cmd = em.get('Commands');
        const cmdName = 'image-editor';
        // Add Image Editor button only if the default command exists
        if (cmd.has(cmdName)) {
          let hasButtonBool = false;
          const tb = this.get('toolbar');
          for (let i = 0; i < tb.length; i++) {
            if (tb[i].command === 'image-editor') {
              hasButtonBool = true;
              break;
            }
          }
          if (!hasButtonBool) {
            tb.push({
              attributes: { class: 'fa fa-pencil' },
              command: cmdName
            });
            this.set('toolbar', tb);
          }
        }
      }
    },
    getAttrToHTML(...args) {
      const attr = Component.prototype.getAttrToHTML.apply(this, args);
      delete attr.onmousedown;
      const src = this.get('src');
      if (src) {
        attr.src = src;
      }
      return attr;
    },
    parseUri(uri) {
      const el = document.createElement('a');
      el.href = uri;
      const query = {};
      const qrs = el.search.substring(1).split('&');
      for (let i = 0; i < qrs.length; i++) {
        const pair = qrs[i].split('=');
        const name = decodeURIComponent(pair[0]);
        if (name) query[name] = decodeURIComponent(pair[1]);
      }
      return {
        hostname: el.hostname,
        pathname: el.pathname,
        protocol: el.protocol,
        search: el.search,
        hash: el.hash,
        port: el.port,
        query
      };
    }
  },
  {
    isComponent(el) {
      let result = '';
      if (el.tagName == 'IMG') {
        result = { type: 'image' };
      }
      return result;
    }
  }
);
