import { on, off } from 'utils/mixins';

const ComponentView = require('./ComponentView');

module.exports = ComponentView.extend({
  events: {
    dblclick: 'onActive',
    input: 'onInput'
  },
  initialize(o) {
    ComponentView.prototype.initialize.apply(this, arguments);
    this.disableEditing = this.disableEditing.bind(this);
    const model = this.model;
    const em = this.em;
    this.listenTo(model, 'focus', this.onActive);
    this.listenTo(model, 'change:content', this.updateContentText);
    this.rte = em && em.get('RichTextEditor');
  },
  updateContentText(m, v, opts = {}) {
    if (!opts.fromDisable) {
      this.disableEditing();
    }
  },
  onActive(e) {
    if (this.rteEnabled || !this.model.get('editable')) {
      return;
    }
    if (e && e.stopPropagation) {
      e.stopPropagation();
    }
    const rte = this.rte;
    if (rte) {
      try {
        this.activeRte = rte.enable(this, this.activeRte);
      } catch (err) {
        console.error(err);
      }
    }
    this.rteEnabled = 1;
    this.toggleEvents(1);
  },
  disableEditing() {
    const model = this.model;
    const editable = model.get('editable');
    const rte = this.rte;
    const contentOpt = { fromDisable: 1 };
    if (rte && editable) {
      try {
        rte.disable(this, this.activeRte);
      } catch (err) {
        console.error(err);
      }
      const content = this.getChildrenContainer().innerHTML;
      const comps = model.get('components');
      if (comps.length) {
        comps.reset();
      }
      model.set('content', '', contentOpt);
      if (rte.customRte) {
        model.set('content', content, contentOpt);
      } else {
        const clean = model => {
          const selectable = !['text', 'default', ''].some(type =>
            model.is(type)
          );
          model.set({
            editable: selectable && model.get('editable'),
            highlightable: 0,
            removable: 0,
            draggable: 0,
            copyable: 0,
            selectable,
            hoverable: selectable,
            toolbar: ''
          });
          model.get('components').each(model => clean(model));
        };
        model.trigger('change:content', model, '', contentOpt);
        comps.add(content);
        comps.each(model => clean(model));
        comps.trigger('resetNavigator');
      }
    }
    this.rteEnabled = 0;
    this.toggleEvents();
  },
  onInput(e) {
    const { em } = this;
    if (em && em.trigger) {
      em.trigger('change:canvasOffset');
    }
  },
  disablePropagation(e) {
    e.stopPropagation();
  },
  toggleEvents(enable) {
    const method = enable ? 'on' : 'off';
    const mixins = { on, off };
    this.em.setEditing(enable);
    const elDocs = [this.el.ownerDocument, document];
    mixins.off(elDocs, 'mousedown', this.disableEditing);
    mixins[method](elDocs, 'mousedown', this.disableEditing);
    this.$el.off('mousedown', this.disablePropagation);
    this.$el[method]('mousedown', this.disablePropagation);
  }
});
