import Backbone from 'backbone';
import { isUndefined } from 'underscore';

module.exports = Backbone.View.extend({
  initialize(o) {
    this.opts = o || {};
    this.config = o.config || {};
    const coll = this.collection;
    this.listenTo(coll, 'add', this.addTo);
    this.listenTo(coll, 'reset', this.resetChildren);
    this.listenTo(coll, 'remove', this.removeChildren);
  },
  removeChildren(removed) {
    const em = this.config.em;
    const view = removed.view;
    const temp = removed.opt.temporary;
    if (!view) return;
    view.remove.apply(view);
    const children = view.childrenView;
    if (children && children.stopListening) {
      children.stopListening();
    }
    removed.components().forEach(this.removeChildren.bind(this));
    if (!temp) {
      removed.removed();
    }
    if (em) {
      if (removed.get('style-signature')) {
        em.get('Commands').run('core:component-style-clear', {
          target: removed
        });
      }
      if (!temp) {
        em.trigger('component:remove', removed);
      }
    }
  },
  addTo(model, coll = {}, opts = {}) {
    const em = this.config.em;
    const i = this.collection.indexOf(model);
    this.addToCollection(model, null, i);
    if (em && !opts.temporary) {
      em.trigger('component:add', model);
    }
  },
  addToCollection(model, fragmentEl, index) {
    if (!this.compView) this.compView = require('./ComponentView');
    const fragment = fragmentEl || null;
    let viewObject = this.compView;
    const dt = this.opts.componentTypes;
    const type = model.get('type');
    for (let it = 0; it < dt.length; it++) {
      const dtId = dt[it].id;
      if (dtId == type) {
        viewObject = dt[it].view;
        break;
      }
    }
    const view = new viewObject({
      model,
      config: this.config,
      componentTypes: dt
    });
    let rendered = view.render().el;
    if (view.model.get('type') == 'textnode')
      rendered = document.createTextNode(view.model.get('content'));
    if (fragment) {
      fragment.appendChild(rendered);
    } else {
      const parent = this.parentEl;
      const children = parent.childNodes;
      if (!isUndefined(index)) {
        const lastIndex = children.length == index;
        if (lastIndex) {
          index--;
        }
        if (lastIndex || !children.length) {
          parent.appendChild(rendered);
        } else {
          parent.insertBefore(rendered, children[index]);
        }
      } else {
        parent.appendChild(rendered);
      }
    }
    return rendered;
  },
  resetChildren() {
    this.parentEl.innerHTML = '';
    this.collection.each(model => this.addToCollection(model));
  },
  render(parent) {
    const el = this.el;
    const frag = document.createDocumentFragment();
    this.parentEl = parent || this.el;
    this.collection.each(model => this.addToCollection(model, frag));
    el.innerHTML = '';
    el.appendChild(frag);
    return this;
  }
});
