import { isString, isArray, keys } from 'underscore';
import { shallowDiff } from 'utils/mixins';
import ParserHtml from 'parser/model/ParserHtml';

const parseStyle = ParserHtml().parseStyle;
export default {
  parseStyle,
  extendStyle(prop) {
    return { ...this.getStyle(), ...prop };
  },
  getStyle() {
    return { ...this.get('style') };
  },
  setStyle(prop = {}, opts = {}) {
    if (isString(prop)) {
      prop = parseStyle(prop);
    }
    const propOrig = this.getStyle();
    const propNew = { ...prop };
    this.set('style', propNew, opts);
    const diff = shallowDiff(propOrig, propNew);
    keys(diff).forEach(pr => {
      const em = this.em;
      this.trigger(`change:style:${pr}`);
      if (em) {
        em.trigger(`styleable:change`, this, pr);
        em.trigger(`styleable:change:${pr}`, this, pr);
      }
    });
    return propNew;
  },
  addStyle(prop, value = '', opts = {}) {
    if (typeof prop === 'string') {
      prop = {
        prop: value
      };
    } else {
      opts = value || {};
    }
    prop = this.extendStyle(prop);
    this.setStyle(prop, opts);
  },
  removeStyle(prop) {
    const style = this.getStyle();
    delete style[prop];
    this.setStyle(style);
  },
  styleToString(opts = {}) {
    const result = [];
    const style = this.getStyle();
    for (const prop in style) {
      const imp = opts.important;
      const important = isArray(imp) ? imp.indexOf(prop) >= 0 : imp;
      const value = `${style[prop]}${important ? ' !important' : ''}`;
      value && result.push(`${prop}:${value};`);
    }
    return result.join('');
  },
  getSelectors() {
    return this.get('selectors') || this.get('classes');
  }
};
