import $ from 'cash-dom';
import Editor from './editor';
import { isElement, isFunction } from 'underscore';
import polyfills from 'utils/polyfills';
import PluginManager from './plugin_manager';

polyfills();
module.exports = (() => {
  const plugins = new PluginManager();
  const editors = [];
  const defaultConfig = {
    autorender: 1,
    plugins: [],
    pluginsOpts: {}
  };
  return {
    $,
    editors,
    plugins,
    version: '<# VERSION #>',
    init(config = {}) {
      const els = config.container;
      if (!els) throw new Error("'container' is required");
      config = { ...defaultConfig, ...config };
      config.el = isElement(els) ? els : document.querySelector(els);
      const editor = new Editor(config).init();
      config.plugins.forEach(pluginId => {
        let plugin = plugins.get(pluginId);
        const plgOptions = config.pluginsOpts[pluginId] || {};
        if (!plugin) {
          const wplg = window[pluginId];
          plugin = wplg && wplg.default ? wplg.default : wplg;
        }
        if (plugin) {
          plugin(editor, plgOptions);
        } else if (isFunction(pluginId)) {
          pluginId(editor, plgOptions);
        } else {
          console.warn(`Plugin ${pluginId} not found`);
        }
      });
      editor.getModel().loadOnStart();
      if (config.autorender) {
        editor.render();
      }
      editors.push(editor);
      return editor;
    }
  };
})();
