module.exports = () => {
  let c = {};
  const defaults = require('./config/config');
  const ModalM = require('./model/Modal');
  const ModalView = require('./view/ModalView');
  let model;
  let modal;

  return {
    name: 'Modal',
    init(config) {
      c = config || {};
      for (const name in defaults) {
        if (!(name in c)) c[name] = defaults[name];
      }
      const ppfx = c.pStylePrefix;
      if (ppfx) c.stylePrefix = ppfx + c.stylePrefix;
      model = new ModalM(c);
      modal = new ModalView({
        model,
        config: c
      });
      return this;
    },
    postRender(view) {
      const el = view.model.getConfig().el || view.el;
      this.render().appendTo(el);
    },
    open(opts = {}) {
      opts.title && this.setTitle(opts.title);
      opts.content && this.setContent(opts.content);
      modal.show();
      return this;
    },
    close() {
      modal.hide();
      return this;
    },
    isOpen() {
      return !!model.get('open');
    },
    setTitle(title) {
      model.set('title', title);
      return this;
    },
    getTitle() {
      return model.get('title');
    },
    setContent(content) {
      model.set('content', ' ');
      model.set('content', content);
      return this;
    },
    getContent() {
      return model.get('content');
    },
    getContentEl() {
      return modal.getContent().get(0);
    },
    getModel() {
      return model;
    },
    render() {
      return modal.render().$el;
    }
  };
};
