module.exports = () => {
  let c = {};
  const defaults = require('./config/config');
  const Panel = require('./model/Panel');
  const Panels = require('./model/Panels');
  const PanelView = require('./view/PanelView');
  const PanelsView = require('./view/PanelsView');
  let panels;
  let PanelsViewObj;

  return {
    name: 'Panels',
    init(config) {
      c = config || {};
      for (const name in defaults) {
        if (!(name in c)) c[name] = defaults[name];
      }
      const ppfx = c.pStylePrefix;
      if (ppfx) c.stylePrefix = ppfx + c.stylePrefix;

      panels = new Panels(c.defaults);
      PanelsViewObj = new PanelsView({
        collection: panels,
        config: c
      });
      return this;
    },
    getPanels() {
      return panels;
    },
    getPanelsEl() {
      return PanelsViewObj.el;
    },
    addPanel(panel) {
      return panels.add(panel);
    },
    removePanel(panel) {
      return panels.remove(panel);
    },
    getPanel(id) {
      const res = panels.where({ id });
      return res.length ? res[0] : null;
    },
    addButton(panelId, button) {
      const pn = this.getPanel(panelId);
      return pn ? pn.get('buttons').add(button) : null;
    },
    removeButton(panelId, button) {
      const pn = this.getPanel(panelId);
      return pn && pn.get('buttons').remove(button);
    },
    getButton(panelId, id) {
      const pn = this.getPanel(panelId);
      if (pn) {
        const res = pn.get('buttons').where({ id });
        return res.length ? res[0] : null;
      }
      return null;
    },
    render() {
      return PanelsViewObj.render().el;
    },
    active() {
      this.getPanels().each(p => {
        p.get('buttons').each(btn => {
          if (btn.get('active')) btn.trigger('updateActive');
        });
      });
    },
    disableButtons() {
      this.getPanels().each(p => {
        p.get('buttons').each(btn => {
          if (btn.get('disable')) btn.trigger('change:disable');
        });
      });
    },
    Panel
  };
};
