import RichTextEditor from './model/RichTextEditor';
import { on, off } from 'utils/mixins';

module.exports = () => {
  let config = {};
  const defaults = require('./config/config');
  let toolbar;
  let actions;
  let lastEl;
  let globalRte;
  const hideToolbar = () => {
    const style = toolbar.style;
    const size = '-1000px';
    style.top = size;
    style.left = size;
    style.display = 'none';
  };
  return {
    customRte: null,
    name: 'RichTextEditor',
    init(opts = {}) {
      config = opts;
      for (const name in defaults) {
        if (!(name in config)) {
          config[name] = defaults[name];
        }
      }
      const ppfx = config.pStylePrefix;
      if (ppfx) {
        config.stylePrefix = ppfx + config.stylePrefix;
      }
      this.pfx = config.stylePrefix;
      actions = config.actions || [];
      toolbar = document.createElement('div');
      toolbar.className = `${ppfx}rte-toolbar ${ppfx}one-bg`;
      globalRte = this.initRte(document.createElement('div'));
      on(toolbar, 'mousedown', e => e.stopPropagation());
      return this;
    },
    postRender(ev) {
      const canvas = ev.model.get('Canvas');
      toolbar.style.pointerEvents = 'all';
      hideToolbar();
      canvas.getToolsEl().appendChild(toolbar);
    },
    initRte(el) {
      const pfx = this.pfx;
      const actionbarContainer = toolbar;
      const actionbar = this.actionbar;
      const actions = this.actions || config.actions;
      const classes = {
        actionbar: `${pfx}actionbar`,
        button: `${pfx}action`,
        active: `${pfx}active`
      };
      const rte = new RichTextEditor({
        el,
        classes,
        actions,
        actionbar,
        actionbarContainer
      });
      globalRte && globalRte.setEl(el);
      if (rte.actionbar) {
        this.actionbar = rte.actionbar;
      }
      if (rte.actions) {
        this.actions = rte.actions;
      }
      return rte;
    },
    add(name, action = {}) {
      action.name = name;
      globalRte.addAction(action, { sync: 1 });
    },
    get(name) {
      let result;
      globalRte.getActions().forEach(action => {
        if (action.name == name) {
          result = action;
        }
      });
      return result;
    },
    getAll() {
      return globalRte.getActions();
    },
    remove(name) {
      const actions = this.getAll();
      const action = this.get(name);

      if (action) {
        const btn = action.btn;
        const index = actions.indexOf(action);
        btn.parentNode.removeChild(btn);
        actions.splice(index, 1);
      }
      return action;
    },
    getToolbarEl() {
      return toolbar;
    },
    udpatePosition() {
      const un = 'px';
      const canvas = config.em.get('Canvas');
      const pos = canvas.getTargetToElementDim(toolbar, lastEl, {
        event: 'rteToolbarPosUpdate'
      });
      if (pos) {
        if (config.adjustToolbar) {
          if (pos.top <= pos.canvasTop) {
            pos.top = pos.elementTop + pos.elementHeight;
          }
        }
        const toolbarStyle = toolbar.style;
        toolbarStyle.top = pos.top + un;
        toolbarStyle.left = pos.left + un;
      }
    },
    enable(view, rte) {
      lastEl = view.el;
      const em = config.em;
      const el = view.getChildrenContainer();
      const customRte = this.customRte;
      toolbar.style.display = '';
      rte = customRte ? customRte.enable(el, rte) : this.initRte(el).enable();
      if (em) {
        setTimeout(this.udpatePosition.bind(this), 0);
        const event = 'change:canvasOffset canvasScroll';
        em.off(event, this.udpatePosition, this);
        em.on(event, this.udpatePosition, this);
        em.trigger('rte:enable', view, rte);
      }
      return rte;
    },
    disable(view, rte) {
      const em = config.em;
      const customRte = this.customRte;
      const el = view.getChildrenContainer();
      if (customRte) {
        customRte.disable(el, rte);
      } else {
        rte && rte.disable();
      }
      hideToolbar();
      if (em && em.trigger) {
        em.trigger('rte:disable', view, rte);
      }
    }
  };
};
