import { isString, isElement, isObject } from 'underscore';

const isId = str => isString(str) && str[0] == '#';

module.exports = config => {
  let c = config || {};
  const defaults = require('./config/config');
  const Selector = require('./model/Selector');
  const Selectors = require('./model/Selectors');
  const ClassTagsView = require('./view/ClassTagsView');
  let selectors;
  let selectorTags;

  return {
    Selector,

    Selectors,
    name: 'SelectorManager',
    getConfig() {
      return c;
    },
    init(conf) {
      c = conf || {};
      for (const name in defaults) {
        if (!(name in c)) c[name] = defaults[name];
      }
      const em = c.em;
      const ppfx = c.pStylePrefix;
      if (ppfx) {
        c.stylePrefix = ppfx + c.stylePrefix;
      }
      selectorTags = new ClassTagsView({
        collection: new Selectors([], { em, config: c }),
        config: c
      });
      selectors = new Selectors(c.selectors);
      selectors.on('add', model => em.trigger('selector:add', model));
      return this;
    },
    postRender() {
      const elTo = this.getConfig().appendTo;
      if (elTo) {
        const el = isElement(elTo) ? elTo : document.querySelector(elTo);
        el.appendChild(this.render([]));
      }
    },
    add(name, opts = {}) {
      if (isObject(name)) {
        opts = name;
      } else {
        opts.name = name;
      }
      if (isId(opts.name)) {
        opts.name = opts.name.substr(1);
        opts.type = Selector.TYPE_ID;
      }
      if (opts.label && !opts.name) {
        opts.name = Selector.escapeName(opts.label);
      }
      const cname = opts.name;
      const selector = cname
        ? this.get(cname, opts.type)
        : selectors.where(opts)[0];
      if (!selector) {
        return selectors.add(opts);
      }
      return selector;
    },
    addClass(classes) {
      const added = [];

      if (isString(classes)) {
        classes = classes.trim().split(' ');
      }
      classes.forEach(name => added.push(selectors.add({ name })));
      return added;
    },
    get(name, type = Selector.TYPE_CLASS) {
      if (isId(name)) {
        name = name.substr(1);
        type = Selector.TYPE_ID;
      }
      return selectors.where({ name, type })[0];
    },
    getAll() {
      return selectors;
    },
    render(selectors) {
      if (selectors) {
        const view = new ClassTagsView({
          collection: new Selectors(selectors),
          config: c
        });
        return view.render().el;
      }
      return selectorTags.render().el;
    }
  };
};
