const Backbone = require('backbone');

const TYPE_CLASS = 1;
const TYPE_ID = 2;

const Selector = Backbone.Model.extend(
  {
    idAttribute: 'name',
    defaults: {
      name: '',
      label: '',
      type: TYPE_CLASS,
      active: true,
      private: false,
      protected: false
    },
    initialize() {
      const name = this.get('name');
      const label = this.get('label');
      if (!name) {
        this.set('name', label);
      } else if (!label) {
        this.set('label', name);
      }
      this.set('name', Selector.escapeName(this.get('name')));
    },
    getFullName() {
      let init = '';
      switch (this.get('type')) {
        case TYPE_CLASS:
          init = '.';
          break;
        case TYPE_ID:
          init = '#';
          break;
      }
      return init + this.get('name');
    }
  },
  {
    TYPE_CLASS,
    TYPE_ID,
    escapeName(name) {
      return `${name}`.trim().replace(/([^a-z0-9\w-\:]+)/gi, '-');
    }
  }
);
module.exports = Selector;
