import { template } from 'underscore';
import Backbone from 'backbone';

const ClassTagView = require('./ClassTagView');

module.exports = Backbone.View.extend({
  template: template(`
  <div id="<%= pfx %>up">
    <div id="<%= pfx %>label"><%= label %></div>
    <div id="<%= pfx %>status-c">
      <span id="<%= pfx %>input-c">
        <div class="<%= ppfx %>field <%= ppfx %>select">
          <span id="<%= ppfx %>input-holder">
            <select id="<%= pfx %>states">
              <option value=""><%= statesLabel %></option>
            </select>
          </span>
          <div class="<%= ppfx %>sel-arrow">
            <div class="<%= ppfx %>d-s-arrow"></div>
          </div>
        </div>
      </span>
    </div>
  </div>
  <div id="<%= pfx %>tags-field" class="<%= ppfx %>field">
    <div id="<%= pfx %>tags-c"></div>
    <input id="<%= pfx %>new" />
    <span id="<%= pfx %>add-tag" class="fa fa-plus"></span>
  </div>
  <div id="<%= pfx %>sel-help">
    <div id="<%= pfx %>label"><%= selectedLabel %></div>
    <div id="<%= pfx %>sel"></div>
    <div style="clear:both"></div>
  </div>`),
  events: {},
  initialize(o = {}) {
    this.config = o.config || {};
    this.pfx = this.config.stylePrefix || '';
    this.ppfx = this.config.pStylePrefix || '';
    this.className = `${this.pfx}tags`;
    this.addBtnId = `${this.pfx}add-tag`;
    this.newInputId = `${this.pfx}new`;
    this.stateInputId = `${this.pfx}states`;
    this.stateInputC = `${this.pfx}input-c`;
    this.states = this.config.states || [];
    this.events[`click #${this.addBtnId}`] = 'startNewTag';
    this.events[`blur #${this.newInputId}`] = 'endNewTag';
    this.events[`keyup #${this.newInputId}`] = 'onInputKeyUp';
    this.events[`change #${this.stateInputId}`] = 'stateChanged';
    this.target = this.config.em;
    this.em = this.target;
    this.listenTo(
      this.getStyleEmitter(),
      'styleManager:update',
      this.componentChanged
    );
    this.listenTo(
      this.target,
      'component:toggled component:update:classes',
      this.componentChanged
    );
    this.listenTo(this.target, 'component:update:classes', this.updateSelector);
    this.listenTo(this.collection, 'add', this.addNew);
    this.listenTo(this.collection, 'reset', this.renderClasses);
    this.listenTo(this.collection, 'remove', this.tagRemoved);
    this.delegateEvents();
  },
  getStyleEmitter() {
    const { em } = this;
    const sm = em && em.get('StyleManager');
    const emitter = sm && sm.getEmitter();
    return emitter || {};
  },
  tagRemoved(model) {
    this.updateStateVis();
  },
  getStateOptions() {
    let strInput = '';
    for (let i = 0; i < this.states.length; i++) {
      strInput += `<option value="${this.states[i].name}">${
        this.states[i].label
      }</option>`;
    }
    return strInput;
  },
  addNew(model) {
    this.addToClasses(model);
  },
  startNewTag(e) {
    this.$addBtn.get(0).style.display = 'none';
    this.$input.show().focus();
  },
  endNewTag(e) {
    this.$addBtn.get(0).style.display = '';
    this.$input.hide().val('');
  },
  onInputKeyUp(e) {
    if (e.keyCode === 13) this.addNewTag(this.$input.val());
    else if (e.keyCode === 27) this.endNewTag();
  },
  componentChanged(target) {
    target = target || this.getTarget();
    this.compTarget = target;
    let validSelectors = [];

    if (target) {
      const state = target.get('state');
      state && this.getStates().val(state);
      const selectors = target.getSelectors();
      validSelectors = selectors.getValid();
    }
    this.collection.reset(validSelectors);
    this.updateStateVis(target);
  },
  getTarget() {
    const targetStyle = this.getStyleEmitter().model;
    return this.target.getSelected();
  },
  updateStateVis(target) {
    const em = this.em;
    const avoidInline = em && em.getConfig('avoidInlineStyle');
    const display = this.collection.length || avoidInline ? 'block' : 'none';
    this.getStatesC().css('display', display);
    this.updateSelector(target);
  },
  updateSelector(target) {
    const { pfx, collection, el } = this;
    const selected = target || this.getTarget();
    this.compTarget = selected;
    if (!selected || !selected.get) return;
    const state = selected.get('state');
    const coll = collection;
    let result = coll.getFullString(selected.getSelectors().getStyleable());
    result =
      result ||
      selected.get('selectorsAdd') ||
      (selected.getId ? `#${selected.getId()}` : '');
    result += state ? `:${state}` : '';
    const elSel = el.querySelector(`#${pfx}sel`);
    elSel && (elSel.innerHTML = result);
  },
  stateChanged(e) {
    if (this.compTarget) {
      this.compTarget.set('state', this.$states.val());
      this.updateSelector();
    }
  },
  addNewTag(label) {
    const target = this.target;
    const component = this.compTarget;
    if (!label.trim()) {
      return;
    }
    if (target) {
      const sm = target.get('SelectorManager');
      const model = sm.add({ label });
      if (component) {
        const compCls = component.getSelectors();
        compCls.add(model);
        this.collection.add(model);
        this.updateStateVis();
      }
    }
    this.endNewTag();
  },
  addToClasses(model, fragmentEl) {
    const fragment = fragmentEl || null;
    const view = new ClassTagView({
      model,
      config: this.config,
      coll: this.collection
    });
    const rendered = view.render().el;
    if (fragment) {
      fragment.appendChild(rendered);
    } else {
      this.getClasses().append(rendered);
    }
    return rendered;
  },
  renderClasses() {
    const frag = document.createDocumentFragment();
    const classes = this.getClasses();
    this.collection.each(model => this.addToClasses(model, frag));
    classes.get(0) && classes.empty().append(frag);
    return this;
  },
  getClasses() {
    if (!this.$classes) this.$classes = this.$el.find(`#${this.pfx}tags-c`);
    return this.$classes;
  },
  getStates() {
    if (!this.$states) this.$states = this.$el.find(`#${this.stateInputId}`);
    return this.$states;
  },
  getStatesC() {
    if (!this.$statesC) this.$statesC = this.$el.find(`#${this.stateInputC}`);
    return this.$statesC;
  },
  render() {
    const ppfx = this.ppfx;
    const config = this.config;
    const $el = this.$el;
    $el.html(
      this.template({
        selectedLabel: config.selectedLabel,
        statesLabel: config.statesLabel,
        label: config.label,
        pfx: this.pfx,
        ppfx: this.ppfx
      })
    );
    this.$input = $el.find(`input#${this.newInputId}`);
    this.$addBtn = $el.find(`#${this.addBtnId}`);
    this.$classes = $el.find(`#${this.pfx}tags-c`);
    this.$states = $el.find(`#${this.stateInputId}`);
    this.$statesC = $el.find(`#${this.stateInputC}`);
    this.$states.append(this.getStateOptions());
    this.renderClasses();
    $el.attr('class', `${this.className} ${ppfx}one-bg ${ppfx}two-color`);
    return this;
  }
});
