import { isElement } from 'underscore';

module.exports = () => {
  let c = {};
  const defaults = require('./config/config');
  const Sectors = require('./model/Sectors');
  const Properties = require('./model/Properties');
  const SectorsView = require('./view/SectorsView');
  let properties;
  let sectors;
  let SectView;

  return {
    name: 'StyleManager',
    getConfig() {
      return c;
    },
    init(config) {
      c = config || {};
      for (const name in defaults) {
        if (!(name in c)) c[name] = defaults[name];
      }
      const ppfx = c.pStylePrefix;
      if (ppfx) c.stylePrefix = ppfx + c.stylePrefix;
      properties = new Properties();
      sectors = new Sectors([], c);
      SectView = new SectorsView({
        collection: sectors,
        target: c.em,
        config: c
      });
      return this;
    },
    onLoad() {
      sectors.add(c.sectors);
    },
    postRender() {
      const elTo = this.getConfig().appendTo;
      if (elTo) {
        const el = isElement(elTo) ? elTo : document.querySelector(elTo);
        el.appendChild(this.render());
      }
    },
    addSector(id, sector) {
      let result = this.getSector(id);
      if (!result) {
        sector.id = id;
        result = sectors.add(sector);
      }
      return result;
    },
    getSector(id) {
      const res = sectors.where({ id });
      return res.length ? res[0] : null;
    },
    removeSector(id) {
      return this.getSectors().remove(this.getSector(id));
    },
    getSectors() {
      return sectors;
    },
    addProperty(sectorId, property) {
      let prop = null;
      const sector = this.getSector(sectorId);
      if (sector) prop = sector.get('properties').add(property);
      return prop;
    },
    getProperty(sectorId, name) {
      let prop = null;
      const sector = this.getSector(sectorId);
      if (sector) {
        prop = sector.get('properties').where({ property: name });
        prop = prop.length == 1 ? prop[0] : prop;
      }
      return prop;
    },
    removeProperty(sectorId, name) {
      const props = this.getProperties(sectorId);
      return props && props.remove(this.getProperty(sectorId, name));
    },
    getProperties(sectorId) {
      let props = null;
      const sector = this.getSector(sectorId);

      if (sector) props = sector.get('properties');

      return props;
    },
    getModelToStyle(model) {
      const em = c.em;
      const classes = model.get('classes');
      const id = model.getId();
      if (em) {
        const config = em.getConfig();
        const um = em.get('UndoManager');
        const cssC = em.get('CssComposer');
        const state = !config.devicePreviewMode ? model.get('state') : '';
        const valid = classes.getStyleable();
        const hasClasses = valid.length;
        const opts = { state };
        let rule;
        um.stop();
        if (hasClasses) {
          const deviceW = em.getCurrentMedia();
          rule = cssC.get(valid, state, deviceW);
          if (!rule) {
            rule = cssC.add(valid, state, deviceW);
            rule.setStyle(model.getStyle());
            model.setStyle({});
          }
        } else if (config.avoidInlineStyle) {
          rule = cssC.getIdRule(id, opts);
          !rule && (rule = cssC.setIdRule(id, {}, opts));
          if (model.is('wrapper')) rule.set('wrapper', 1);
        }
        rule && (model = rule);
        um.start();
      }
      return model;
    },
    addType(id, definition) {
      properties.addType(id, definition);
    },
    getType(id) {
      return properties.getType(id);
    },
    getTypes() {
      return properties.getTypes();
    },
    createType(id, { model = {}, view = {} } = {}) {
      const type = this.getType(id);
      if (type) {
        return new type.view({
          model: new type.model(model),
          config: c,
          ...view
        });
      }
    },
    setTarget(target, opts) {
      return SectView.setTarget(target, opts);
    },
    getEmitter() {
      return SectView.propTarget;
    },
    render() {
      return SectView.render().el;
    }
  };
};
