import { isUndefined, isString } from 'underscore';

const Property = require('backbone').Model.extend(
  {
    defaults: {
      name: '',
      property: '',
      type: '',
      defaults: '',
      info: '',
      value: '',
      icon: '',
      functionName: '',
      status: '',
      visible: true,
      fixedValues: ['initial', 'inherit'],
      full: 0,
      important: 0,
      toRequire: 0
    },
    initialize(props = {}, opts = {}) {
      const name = this.get('name');
      const prop = this.get('property');
      !this.get('id') && this.set('id', prop);
      if (!name) {
        this.set(
          'name',
          prop.charAt(0).toUpperCase() + prop.slice(1).replace(/-/g, ' ')
        );
      }
      Property.callInit(this, props, opts);
    },
    init() {},
    clearValue(opts = {}) {
      this.set({ value: undefined }, opts);
      return this;
    },
    setValue(value, complete = 1, opts = {}) {
      const parsed = this.parseValue(value);
      this.set(parsed, { ...opts, avoidStore: 1 });
      if (complete) {
        this.set('value', '', opts);
        this.set(parsed, opts);
      }
    },
    setValueFromInput(value, complete, opts = {}) {
      this.setValue(value, complete, { ...opts, fromInput: 1 });
    },
    parseValue(value, opts = {}) {
      const result = { value };
      const imp = '!important';
      if (isString(value) && value.indexOf(imp) !== -1) {
        result.value = value.replace(imp, '').trim();
        result.important = 1;
      }
      if (!this.get('functionName') && !opts.complete) {
        return result;
      }
      const args = [];
      const valueStr = `${result.value}`;
      const start = valueStr.indexOf('(') + 1;
      const end = valueStr.lastIndexOf(')');
      const functionName = valueStr.substring(0, start - 1);
      if (functionName) result.functionName = functionName;
      args.push(start);
      if (end >= 0) {
        args.push(end);
      }
      result.value = String.prototype.substring.apply(valueStr, args);
      if (opts.numeric) {
        const num = parseFloat(result.value);
        result.unit = result.value.replace(num, '');
        result.value = num;
      }
      return result;
    },
    getDefaultValue() {
      return this.get('defaults');
    },
    getFullValue(val) {
      const fn = this.get('functionName');
      let value = isUndefined(val) ? this.get('value') : val;
      if (fn && !isUndefined(value)) {
        value = `${fn}(${value})`;
      }
      if (this.get('important')) {
        value = `${value} !important`;
      }
      return value || '';
    }
  },
  {
    callParentInit(property, ctx, props, opts = {}) {
      property.prototype.initialize.apply(ctx, [
        props,
        {
          ...opts,
          skipInit: 1
        }
      ]);
    },

    callInit(context, props, opts = {}) {
      !opts.skipInit && context.init(props, opts);
    }
  }
);

module.exports = Property;
