import { extend } from 'underscore';

const Backbone = require('backbone');
const Properties = require('./Properties');
const PropertyFactory = require('./PropertyFactory');

module.exports = Backbone.Model.extend({
  defaults: {
    id: '',
    name: '',
    open: true,
    buildProps: '',
    extendBuilded: 1,
    properties: []
  },
  initialize(opts) {
    const o = opts || {};
    let props = [];
    const builded = this.buildProperties(o.buildProps);
    !this.get('id') && this.set('id', this.get('name'));
    if (!builded) {
      props = this.get('properties');
    } else {
      props = this.extendProperties(builded);
    }
    const propsModel = new Properties(props);
    propsModel.sector = this;
    this.set('properties', propsModel);
  },
  extendProperties(props, moProps, ex) {
    const pLen = props.length;
    const mProps = moProps || this.get('properties');
    const ext = this.get('extendBuilded');
    const isolated = [];
    for (let i = 0, len = mProps.length; i < len; i++) {
      const mProp = mProps[i];
      let found = 0;
      for (let j = 0; j < pLen; j++) {
        const prop = props[j];
        if (mProp.property == prop.property || mProp.id == prop.property) {
          const mPProps = mProp.properties;
          if (mPProps && mPProps.length) {
            mProp.properties = this.extendProperties(
              prop.properties || [],
              mPProps,
              1
            );
          }
          props[j] = ext ? extend(prop, mProp) : mProp;
          isolated[j] = props[j];
          found = 1;
          continue;
        }
      }
      if (!found) {
        props.push(mProp);
        isolated.push(mProp);
      }
    }
    return ex ? isolated : props;
  },
  buildProperties(props) {
    let r;
    const buildP = props || [];
    if (!buildP.length) {
      return;
    }
    if (!this.propFactory) {
      this.propFactory = new PropertyFactory();
    }
    r = this.propFactory.build(buildP);
    return r;
  }
});
