const Backbone = require('backbone');
const LayerView = require('./LayerView');

module.exports = Backbone.View.extend({
  initialize(o) {
    this.config = o.config || {};
    this.stackModel = o.stackModel;
    this.preview = o.preview;
    this.pfx = this.config.stylePrefix || '';
    this.ppfx = this.config.pStylePrefix || '';
    this.propsConfig = o.propsConfig;
    const pfx = this.pfx;
    const ppfx = this.ppfx;
    const collection = this.collection;
    this.className = `${pfx}layers ${ppfx}field`;
    this.listenTo(collection, 'add', this.addTo);
    this.listenTo(collection, 'deselectAll', this.deselectAll);
    this.listenTo(collection, 'reset', this.render);
    const em = this.config.em || '';
    const utils = em ? em.get('Utils') : '';
    this.sorter = utils
      ? new utils.Sorter({
          container: this.el,
          ignoreViewChildren: 1,
          containerSel: `.${pfx}layers`,
          itemSel: `.${pfx}layer`,
          pfx: this.config.pStylePrefix
        })
      : '';
    collection.view = this;
    this.$el.data('model', collection);
    this.$el.data('collection', collection);
  },
  addTo(model) {
    const i = this.collection.indexOf(model);
    this.addToCollection(model, null, i);
  },
  addToCollection(model, fragmentEl, index) {
    const fragment = fragmentEl || null;
    const stackModel = this.stackModel;
    const config = this.config;
    const sorter = this.sorter;
    const propsConfig = this.propsConfig;
    if (typeof this.preview !== 'undefined') {
      model.set('preview', this.preview);
    }
    const view = new LayerView({
      model,
      config,
      sorter,
      stackModel,
      propsConfig
    });
    const rendered = view.render().el;
    if (fragment) {
      fragment.appendChild(rendered);
    } else if (typeof index !== 'undefined') {
      let method = 'before';
      if (this.$el.children().length == index) {
        index--;
        method = 'after';
      }
      if (index < 0) {
        this.$el.append(rendered);
      } else {
        this.$el
          .children()
          .eq(index)
          [method](rendered);
      }
    } else {
      this.$el.append(rendered);
    }

    return rendered;
  },
  deselectAll() {
    this.$el.find(`.${this.pfx}layer`).removeClass(`${this.pfx}active`);
  },
  render() {
    const fragment = document.createDocumentFragment();
    this.$el.empty();
    this.collection.each(function(model) {
      this.addToCollection(model, fragment);
    }, this);
    this.$el.append(fragment);
    this.$el.attr('class', this.className);
    if (this.sorter) {
      this.sorter.plh = null;
    }
    return this;
  }
});
