import _ from 'underscore';
import Backbone from 'backbone';

const PropertiesView = require('./PropertiesView');

module.exports = Backbone.View.extend({
  template: _.template(`
  <div class="<%= pfx %>title" data-sector-title>
    <i id="<%= pfx %>caret" class="fa"></i>
    <%= label %>
  </div>`),
  events: {
    'click [data-sector-title]': 'toggle'
  },
  initialize(o) {
    this.config = o.config || {};
    this.pfx = this.config.stylePrefix || '';
    this.target = o.target || {};
    this.propTarget = o.propTarget || {};
    this.caretR = 'fa-caret-right';
    this.caretD = 'fa-caret-down';
    const model = this.model;
    this.listenTo(model, 'change:open', this.updateOpen);
    this.listenTo(model, 'updateVisibility', this.updateVisibility);
    this.listenTo(model, 'destroy remove', this.remove);
  },
  updateVisibility() {
    let show;
    this.model.get('properties').each(prop => {
      if (prop.get('visible')) {
        show = 1;
      }
    });
    this.el.style.display = show ? 'block' : 'none';
  },
  updateOpen() {
    if (this.model.get('open')) this.show();
    else this.hide();
  },
  show() {
    this.$el.addClass(`${this.pfx}open`);
    this.getPropertiesEl().style.display = '';
    this.$caret.removeClass(this.caretR).addClass(this.caretD);
  },
  hide() {
    this.$el.removeClass(`${this.pfx}open`);
    this.getPropertiesEl().style.display = 'none';
    this.$caret.removeClass(this.caretD).addClass(this.caretR);
  },
  getPropertiesEl() {
    return this.$el.find(`.${this.pfx}properties`).get(0);
  },
  toggle(e) {
    const v = this.model.get('open') ? 0 : 1;
    this.model.set('open', v);
  },

  render() {
    this.$el.html(
      this.template({
        pfx: this.pfx,
        label: this.model.get('name')
      })
    );
    this.$caret = this.$el.find(`#${this.pfx}caret`);
    this.renderProperties();
    this.$el.attr('class', `${this.pfx}sector no-select`);
    this.updateOpen();
    return this;
  },

  renderProperties() {
    const objs = this.model.get('properties');

    if (objs) {
      const view = new PropertiesView({
        collection: objs,
        target: this.target,
        propTarget: this.propTarget,
        config: this.config
      });
      this.$el.append(view.render().el);
    }
  }
});
