import Backbone from 'backbone';

const $ = Backbone.$;

const getBoundingRect = (el, win) => {
  const w = win || window;
  const rect = el.getBoundingClientRect();
  return {
    left: rect.left + w.pageXOffset,
    top: rect.top + w.pageYOffset,
    width: rect.width,
    height: rect.height
  };
};

module.exports = {
  // TODO move to opts
  setKey(keys, command) {
    // key(keys, command);
  },
  getElementRect(el) {
    const posFetcher = this.opts.posFetcher || '';
    return posFetcher
      ? posFetcher(el, {
          avoidFrameOffset: 1
        })
      : getBoundingRect(el);
  },
  init(opts) {
    this.setOptions(opts);
    this.handleMouseDown = this.handleMouseDown.bind(this);
    this.drag = this.drag.bind(this);
    this.move = this.move.bind(this);
    this.stop = this.stop.bind(this);
    this.setKey('up, right, down, left', this.handleKey);
    return this;
  },
  setOptions(opts) {
    this.opts = opts || {};
  },
  focus(el) {
    // Avoid focusing on already focused element
    if (el && el === this.el) {
      return;
    }
    this.getDocumentEl(el);
    this.blur();
    this.el = el;
    this.handlers = this.opts.dragHandlers || [el];
    const elRect = this.getElementRect(el); // <-- TODO have wrong top:left
    this.elRect = elRect;
    this.startTop = elRect.top;
    this.startLeft = elRect.left;
    this.getDocumentEl().on('mousedown', this.handleMouseDown);
  },
  blur() {
    this.getDocumentEl().off('mousedown', this.handleMouseDown);
    this.el = null;
  },
  start(e) {
    this.startPos = this.getMousePos(e);
    const docs = this.getDocumentEl();
    docs.on('mousemove', this.drag);
    docs.on('mouseup', this.stop);
    const onStart = this.opts.onStart;
    if (typeof onStart === 'function') {
      onStart(e, {
        docs,
        el: this.el,
        start: this.startPos,
        elRect: this.elRect
      });
    }
    this.drag(e);
  },
  stop(e) {
    const docs = this.getDocumentEl();
    docs.off('mousemove', this.drag);
    docs.off('mouseup', this.stop);
    this.lockedAxis = null;
    const onEnd = this.opts.onEnd;
    if (typeof onEnd === 'function') {
      onEnd(e, {
        docs,
        delta: this.delta,
        end: {
          x: this.startLeft + this.delta.x,
          y: this.startTop + this.delta.y
        }
      });
    }
  },
  handleMouseDown(e) {
    const el = e.target;
    if (this.isHandler(el)) {
      this.start(e);
    }
  },
  isHandler(el) {
    const handlers = this.handlers;
    for (const n in handlers) {
      if (handlers[n] === el) return true;
    }
    return false;
  },
  handleKey(e, handler) {
    switch (handler.shortcut) {
      case 'up':
        this.move(0, -1);
        break;
      case 'right':
        this.move(1, 0);
        break;
      case 'down':
        this.move(0, 1);
        break;
      case 'left':
        this.move(-1, 0);
        break;
    }
  },
  getDocumentEl(el) {
    var el = el || this.el;
    if (!this.$doc) {
      const docs = [document];
      if (el) {
        docs.push(el.ownerDocument);
      }
      this.$doc = $(docs);
    }
    return this.$doc;
  },
  getMousePos(e) {
    const mouseFetch = this.opts.mousePosFetcher;
    return mouseFetch
      ? mouseFetch(e)
      : {
          x: e.clientX,
          y: e.clientY
        };
  },
  drag(e) {
    let lockedAxis = this.lockedAxis;
    const currentPos = this.getMousePos(e);
    const delta = {
      x: currentPos.x - this.startPos.x,
      y: currentPos.y - this.startPos.y
    };
    if (e.shiftKey) {
      if (!lockedAxis) {
        const relX = delta.x;
        const relY = delta.y;
        const absX = Math.abs(relX);
        const absY = Math.abs(relY);
        if (relY >= absX || relY <= -absX) {
          lockedAxis = 'x';
        } else if (relX > absY || relX < -absY) {
          lockedAxis = 'y';
        }
      }
    } else {
      lockedAxis = null;
    }
    if (lockedAxis === 'x') {
      delta.x = this.startPos.x;
    }
    if (lockedAxis === 'y') {
      delta.y = this.startPos.y;
    }
    this.lockedAxis = lockedAxis;
    this.delta = delta;
    this.move(delta.x, delta.y);
    const onDrag = this.opts.onDrag;
    if (typeof onDrag === 'function') {
      onDrag(e, {
        delta,
        current: {
          x: this.startLeft + delta.x,
          y: this.startTop + delta.y
        },
        lockedAxis
      });
    }
    if (e.which === 0) {
      this.stop(e);
    }
  },
  move(x, y) {
    this.moveX(x);
    this.moveY(y);
  },
  moveX(x) {
    const el = this.el;
    const opts = this.opts;
    const xPos = this.startLeft + x;
    const setX = this.opts.setX;
    if (typeof setX === 'function') {
      setX(xPos, {
        el,
        start: this.startLeft,
        delta: x
      });
    } else {
      el.style.left = `${xPos}px`;
    }
  },
  moveY(y) {
    const el = this.el;
    const opts = this.opts;
    const yPos = this.startTop + y;
    const setY = this.opts.setY;
    if (typeof setY === 'function') {
      setY(yPos, {
        el,
        start: this.startTop,
        delta: y
      });
    } else {
      el.style.top = `${yPos}px`;
    }
  }
};
